/****************************************************************
 UnUltimate: Reverse the UltimateIRCd 3.0.x cloaking system.
 Copyright 2005 Hacker Factor.  All rights reserved.
 Not for commercial use.

 This system uses MD5 to hash IP elements.
 Fortunately, they do not use any type of random number seed or
 chaining, so there are only 256 hashes, and each hash is different.

 They use 2 different algorithms for hashing.
 For IP addresses (where the IP address cannot be resolved to a hostname),
 they simply hash each of the octets.  The octets are then displayed
 in reverse order:
   a.b.c.d = MD5(d).MD5(c).MD5(b).MD5(a)
 For example:
   ./UnUltimate seekip f021f9f.1c6ad229.3f38bee0.cc59707X
   24.144.38.79

 For hostnames, they only hash the last 2 octets, then append the
 last three sections of the domain name.
 As an example:
   ./UnUltimate seekdomain 18c3868a.2142b218.adsl-line.inode.at
   (a.b.150.61) in adsl-line.inode.at
 Although we do not know the full IP address, the domain leads to
 the subnet (a.b).  And we know the host within the subnet (c.d).
 In some cases (where the full hostname+domain is more than 4 segments)
 and where the ISP spans multiple class-B subnets, we may not be able
 to identify the specific IP address, but we can still narrow the search
 space to a few likely systems.

 Unfortunately, there is likely ambiguity from domain names since the
 (a.b) is unknown and many ISPs span class-B and class-C subnets.
 Ways to remove ambiguity:
  - Search Google for other hosts with the domain name.
    Scan the subnet ranges for those hosts.
  - Lookup all 65536 IP combinations (a.b) and look for a matching domain.
    - If you know the country (".uk", ".ni", etc.) then restrict the
      search to all class-B that map to that country.
    - If you know the "a", then you only need to scan 256 "b".
      (E.g., anything ".comcast.net" is likely "24.", "67.", or "68.".)
 - For each match, you can do a simple ping or port scan to see if the host
   is "up".  (Up or Filtered = live host.  A dead host can be likely
   ruled out.)


 To compile:
   cc UnUltimate.c -o UnUltimate

 Run it without parameters to see the usage.
 ****************************************************************/

/***************************************************
 This code is based on:

 *  UltimateIRCd - an Internet Relay Chat Daemon, src/s_hidehost.c
 *
 *  Copyright (C) 1990-2005 by the past and present ircd coders, and others.
 *  Refer to the documentation within doc/authors/ for full credits and copyrights.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 *  USA
 *
 *  $Id: s_hidehost.c 904 2005-04-06 02:34:11Z shadowmaster $

 BUT: All source code from s_hidehost.c was rewritten, so this copyright
 does not apply to UnUltimate.c.  The exception is the CRC function,
 which is not copyright by UltimateIRCd and not GPL.  (In fact,
 proper credit was not given for the modification to the CRC API.)
 The CRC function has a history that long-predates UltimateIRCd.
 The CRC function is considered open source and not GPL.
 ********************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/stat.h>
#include <fcntl.h>
/* for gethostbyname() */ 
#include <netdb.h>
#include <sys/socket.h>
/* for inet_addr() */
#include <netinet/in.h>
#include <arpa/inet.h>


#define HOSTLEN 65

/* Hidden host code below */

#define KEY	23857
#define KEY2	38447
#define KEY3	64709

  /* ============================================================= */
  /*  COPYRIGHT (C) 1986 Gary S. Brown.  You may use this program, or       */
  /*  code or tables extracted from it, as desired without restriction.     */
  /*                                                                        */
  /*  First, the polynomial itself and its table of feedback terms.  The    */
  /*  polynomial is                                                         */
  /*  X^32+X^26+X^23+X^22+X^16+X^12+X^11+X^10+X^8+X^7+X^5+X^4+X^2+X^1+X^0   */
  /*                                                                        */
  /*  Note that we take it "backwards" and put the highest-order term in    */
  /*  the lowest-order bit.  The X^32 term is "implied"; the LSB is the     */
  /*  X^31 term, etc.  The X^0 term (usually shown as "+1") results in      */
  /*  the MSB being 1.                                                      */
  /*                                                                        */
  /*  Note that the usual hardware shift register implementation, which     */
  /*  is what we're using (we're merely optimizing it by doing eight-bit    */
  /*  chunks at a time) shifts bits into the lowest-order term.  In our     */
  /*  implementation, that means shifting towards the right.  Why do we     */
  /*  do it this way?  Because the calculated CRC must be transmitted in    */
  /*  order from highest-order term to lowest-order term.  UARTs transmit   */
  /*  characters in order from LSB to MSB.  By storing the CRC this way,    */
  /*  we hand it to the UART in the order low-byte to high-byte; the UART   */
  /*  sends each low-bit to hight-bit; and the result is transmission bit   */
  /*  by bit from highest- to lowest-order term without requiring any bit   */
  /*  shuffling on our part.  Reception works similarly.                    */
  /*                                                                        */
  /*  The feedback terms table consists of 256, 32-bit entries.  Notes:     */
  /*                                                                        */
  /*      The table can be generated at runtime if desired; code to do so   */
  /*      is shown later.  It might not be obvious, but the feedback        */
  /*      terms simply represent the results of eight shift/xor opera-      */
  /*      tions for all combinations of data and CRC register values.       */
  /*                                                                        */
  /*      The values must be right-shifted by eight bits by the "updcrc"    */
  /*      logic; the shift must be unsigned (bring in zeroes).  On some     */
  /*      hardware you could probably optimize the shift in assembler by    */
  /*      using byte-swap instructions.                                     */
  /*      polynomial $edb88320                                              */
  /*                                                                        */
  /*  --------------------------------------------------------------------  */

static unsigned long crc32_tab[] = {
  0x00000000L, 0x77073096L, 0xee0e612cL, 0x990951baL, 0x076dc419L,
  0x706af48fL, 0xe963a535L, 0x9e6495a3L, 0x0edb8832L, 0x79dcb8a4L,
  0xe0d5e91eL, 0x97d2d988L, 0x09b64c2bL, 0x7eb17cbdL, 0xe7b82d07L,
  0x90bf1d91L, 0x1db71064L, 0x6ab020f2L, 0xf3b97148L, 0x84be41deL,
  0x1adad47dL, 0x6ddde4ebL, 0xf4d4b551L, 0x83d385c7L, 0x136c9856L,
  0x646ba8c0L, 0xfd62f97aL, 0x8a65c9ecL, 0x14015c4fL, 0x63066cd9L,
  0xfa0f3d63L, 0x8d080df5L, 0x3b6e20c8L, 0x4c69105eL, 0xd56041e4L,
  0xa2677172L, 0x3c03e4d1L, 0x4b04d447L, 0xd20d85fdL, 0xa50ab56bL,
  0x35b5a8faL, 0x42b2986cL, 0xdbbbc9d6L, 0xacbcf940L, 0x32d86ce3L,
  0x45df5c75L, 0xdcd60dcfL, 0xabd13d59L, 0x26d930acL, 0x51de003aL,
  0xc8d75180L, 0xbfd06116L, 0x21b4f4b5L, 0x56b3c423L, 0xcfba9599L,
  0xb8bda50fL, 0x2802b89eL, 0x5f058808L, 0xc60cd9b2L, 0xb10be924L,
  0x2f6f7c87L, 0x58684c11L, 0xc1611dabL, 0xb6662d3dL, 0x76dc4190L,
  0x01db7106L, 0x98d220bcL, 0xefd5102aL, 0x71b18589L, 0x06b6b51fL,
  0x9fbfe4a5L, 0xe8b8d433L, 0x7807c9a2L, 0x0f00f934L, 0x9609a88eL,
  0xe10e9818L, 0x7f6a0dbbL, 0x086d3d2dL, 0x91646c97L, 0xe6635c01L,
  0x6b6b51f4L, 0x1c6c6162L, 0x856530d8L, 0xf262004eL, 0x6c0695edL,
  0x1b01a57bL, 0x8208f4c1L, 0xf50fc457L, 0x65b0d9c6L, 0x12b7e950L,
  0x8bbeb8eaL, 0xfcb9887cL, 0x62dd1ddfL, 0x15da2d49L, 0x8cd37cf3L,
  0xfbd44c65L, 0x4db26158L, 0x3ab551ceL, 0xa3bc0074L, 0xd4bb30e2L,
  0x4adfa541L, 0x3dd895d7L, 0xa4d1c46dL, 0xd3d6f4fbL, 0x4369e96aL,
  0x346ed9fcL, 0xad678846L, 0xda60b8d0L, 0x44042d73L, 0x33031de5L,
  0xaa0a4c5fL, 0xdd0d7cc9L, 0x5005713cL, 0x270241aaL, 0xbe0b1010L,
  0xc90c2086L, 0x5768b525L, 0x206f85b3L, 0xb966d409L, 0xce61e49fL,
  0x5edef90eL, 0x29d9c998L, 0xb0d09822L, 0xc7d7a8b4L, 0x59b33d17L,
  0x2eb40d81L, 0xb7bd5c3bL, 0xc0ba6cadL, 0xedb88320L, 0x9abfb3b6L,
  0x03b6e20cL, 0x74b1d29aL, 0xead54739L, 0x9dd277afL, 0x04db2615L,
  0x73dc1683L, 0xe3630b12L, 0x94643b84L, 0x0d6d6a3eL, 0x7a6a5aa8L,
  0xe40ecf0bL, 0x9309ff9dL, 0x0a00ae27L, 0x7d079eb1L, 0xf00f9344L,
  0x8708a3d2L, 0x1e01f268L, 0x6906c2feL, 0xf762575dL, 0x806567cbL,
  0x196c3671L, 0x6e6b06e7L, 0xfed41b76L, 0x89d32be0L, 0x10da7a5aL,
  0x67dd4accL, 0xf9b9df6fL, 0x8ebeeff9L, 0x17b7be43L, 0x60b08ed5L,
  0xd6d6a3e8L, 0xa1d1937eL, 0x38d8c2c4L, 0x4fdff252L, 0xd1bb67f1L,
  0xa6bc5767L, 0x3fb506ddL, 0x48b2364bL, 0xd80d2bdaL, 0xaf0a1b4cL,
  0x36034af6L, 0x41047a60L, 0xdf60efc3L, 0xa867df55L, 0x316e8eefL,
  0x4669be79L, 0xcb61b38cL, 0xbc66831aL, 0x256fd2a0L, 0x5268e236L,
  0xcc0c7795L, 0xbb0b4703L, 0x220216b9L, 0x5505262fL, 0xc5ba3bbeL,
  0xb2bd0b28L, 0x2bb45a92L, 0x5cb36a04L, 0xc2d7ffa7L, 0xb5d0cf31L,
  0x2cd99e8bL, 0x5bdeae1dL, 0x9b64c2b0L, 0xec63f226L, 0x756aa39cL,
  0x026d930aL, 0x9c0906a9L, 0xeb0e363fL, 0x72076785L, 0x05005713L,
  0x95bf4a82L, 0xe2b87a14L, 0x7bb12baeL, 0x0cb61b38L, 0x92d28e9bL,
  0xe5d5be0dL, 0x7cdcefb7L, 0x0bdbdf21L, 0x86d3d2d4L, 0xf1d4e242L,
  0x68ddb3f8L, 0x1fda836eL, 0x81be16cdL, 0xf6b9265bL, 0x6fb077e1L,
  0x18b74777L, 0x88085ae6L, 0xff0f6a70L, 0x66063bcaL, 0x11010b5cL,
  0x8f659effL, 0xf862ae69L, 0x616bffd3L, 0x166ccf45L, 0xa00ae278L,
  0xd70dd2eeL, 0x4e048354L, 0x3903b3c2L, 0xa7672661L, 0xd06016f7L,
  0x4969474dL, 0x3e6e77dbL, 0xaed16a4aL, 0xd9d65adcL, 0x40df0b66L,
  0x37d83bf0L, 0xa9bcae53L, 0xdebb9ec5L, 0x47b2cf7fL, 0x30b5ffe9L,
  0xbdbdf21cL, 0xcabac28aL, 0x53b39330L, 0x24b4a3a6L, 0xbad03605L,
  0xcdd70693L, 0x54de5729L, 0x23d967bfL, 0xb3667a2eL, 0xc4614ab8L,
  0x5d681b02L, 0x2a6f2b94L, 0xb40bbe37L, 0xc30c8ea1L, 0x5a05df1bL,
  0x2d02ef8dL
};

/* Return a 32-bit CRC of the contents of the buffer. */

unsigned long	crc32	(const unsigned char *s, unsigned int len)
{
  unsigned int i;
  unsigned long crc32val;

  crc32val = 0;
  for (i = 0; i < len; i++)
    {
    crc32val = crc32_tab[(crc32val ^ s[i]) & 0xff] ^ (crc32val >> 8);
    }
  return crc32val;
}


/***************************************************
 OctetLookup(): Given an octet index, return the
 encoded string.
 Why it works:  The cloaking system does not chain
 values.  Each octet is encoded independently.
 ***************************************************/
char *	OctetLookup	(int Octet)
{
  static char OctetString[10];
  unsigned long Hash;

  memset(OctetString,'\0',sizeof(OctetString));
  sprintf(OctetString,"%d",Octet);
  Hash = ((crc32(OctetString,strlen(OctetString)) + KEY2) ^ KEY) ^ KEY3;
  Hash <<= 2;
  Hash >>= 2;
  memset(OctetString,'\0',sizeof(OctetString));
  snprintf(OctetString,9,"%lx",Hash);
  return(OctetString);
} /* OctetLookup() */


/****************************************************
 HideHost(): Encode a host.
 Based on make_virthost() from s_hidden.c, UltimateIRC 3.0.1.
 NOTE: The original algorithm has a potential read-overflow
 if the host name only contains one segment (the TLD).
 The only example I know: "ws".
 ****************************************************/
void	HideHost	(char *HostIp)
{
  char *PartsIP[4], *Domain;
  char IP[HOSTLEN+1];
  char Host[HOSTLEN+1];
  int i;
  struct hostent *HostEnt=NULL;
  struct in_addr AddrIn;

  /* Need both IP and hostname to run */
  sethostent(1);
  if (inet_aton(HostIp,&AddrIn))
	HostEnt = gethostbyaddr(&AddrIn,sizeof(AddrIn),AF_INET);
  if (!HostEnt) HostEnt = gethostbyname(HostIp);
  if (HostEnt)
    {
    memset(Host,0,sizeof(Host));
    memset(IP,0,sizeof(IP));
    sprintf(IP,"%d.%d.%d.%d",
	(int)(HostEnt->h_addr[0])&0xff,(int)(HostEnt->h_addr[1])&0xff,
	(int)(HostEnt->h_addr[2])&0xff,(int)(HostEnt->h_addr[3])&0xff);
    strcpy(Host,HostEnt->h_name);
    }
  else
    {
    fprintf(stderr,"ERROR: Unknown host \"%s\"\n",HostIp);
    exit(-1);
    }

  printf("Encoding: %s  %s\n",IP,Host);
  endhostent();

  /* Paranoia. ip & host are pointers */
  PartsIP[0]=IP;
  if (PartsIP[0]) PartsIP[1]=strchr(PartsIP[0],'.');
  else PartsIP[1]=NULL;
  if (PartsIP[1]) PartsIP[1]++;
  if (PartsIP[1]) PartsIP[2]=strchr(PartsIP[1],'.');
  else PartsIP[2]=NULL;
  if (PartsIP[2]) PartsIP[2]++;
  if (PartsIP[2]) PartsIP[3]=strchr(PartsIP[2],'.');
  else PartsIP[3]=NULL;
  if (PartsIP[3]) PartsIP[3]++;

  /* Search for the domain name (no longer than 3 segments) */
  for(i=strlen(Host)-1; (i>=0) && (Host[i] != '.'); i--) ;
  Domain = Host+i+1;
  for(i-- ; (i>=0) && (Host[i] != '.'); i--) ;
  if (Host[i] == '.') Domain = Host+i+1;
  else Domain = Host;
  for(i-- ; (i>=0) && (Host[i] != '.'); i--) ;
  if (Host[i] == '.') Domain = Host+i+1;

  /* If it's an IP address... */
  if (inet_aton(HostIp,&AddrIn) != 0)
      {
      /* print as 4 separate lines since OctetLookup returns a static */
      printf("%s",OctetLookup(atoi(PartsIP[3])));
      printf(".%s",OctetLookup(atoi(PartsIP[2])));
      printf(".%s",OctetLookup(atoi(PartsIP[1])));
      printf(".%sX",OctetLookup(atoi(PartsIP[0])));
      }
  /* If it's a hostname with a domain */
  else
      {
      printf("%s",OctetLookup(atoi(PartsIP[3])));
      printf(".%s",OctetLookup(atoi(PartsIP[2])));
      printf(".%s",Domain);
      }
  printf("\n");
} /* HideHost() */

/***************************************************
 SeekIp(): Given a cloaked IP address, determine the
 uncloaked value(s).
 How it works: Each octet is encoded independently.
 "1.2.3.4" = 21b52b10.3fd056b1.8d7465b.11de379dX
 "4.3.2.1" = 11de379d.8d7465b.3fd056b1.21b52b10X
 Since the hashes never change, we can just look them up.
 ***************************************************/
void	SeekIp	(char *IpString)
{
  char *Parts[4];
  int a,b,c,d;
  char *Hidden;

  /* seek octets */
  Parts[0]=IpString;
  Parts[1]=strchr(Parts[0],'.');
  Parts[1]++;
  Parts[2]=strchr(Parts[1],'.');
  Parts[2]++;
  Parts[3]=strchr(Parts[2],'.');
  Parts[3]++;

  /* brute-force compare */
  for(a=0; a<256; a++)
    {
    Hidden = OctetLookup(a);
    if (!strncasecmp(Parts[3],Hidden,strlen(Hidden)))
      {
      for(b=0; b<256; b++)
	{
	Hidden = OctetLookup(b);
	if (!strncasecmp(Parts[2],Hidden,strlen(Hidden)))
	  {
	  for(c=0; c<256; c++)
	    {
	    Hidden = OctetLookup(c);
	    if (!strncasecmp(Parts[1],Hidden,strlen(Hidden)))
	      {
	      for(d=0; d<256; d++)
		{
		Hidden = OctetLookup(d);
		if (!strncasecmp(Parts[0],Hidden,strlen(Hidden)))
		  {
		  printf("%d.%d.%d.%d\n",a,b,c,d);
		  }
		} /* for d */
	      } /* if match c */
	    } /* for c */
	  } /* if match b */
	} /* for b */
      } /* if match a */
    } /* for a */
} /* SeekIp() */

/***************************************************
 SeekHost(): Given a cloaked hostname, determine the
 uncloaked value(s).
 How it works:
 The cloaked hostname is based on the hostname and
 IP address (a.b.c.d).
   - Remove the host (just keep domain).
   - Create: cloak(d).cloak(c).domain
 How to reverse it:
   - Given a domain name, find the subnet.
     The subnet defines "a.b.".
   - Check if "a.b.c.d" matches.
 ***************************************************/
void	SeekHost	(char *HostString)
{
  char *Parts[4];
  int c,d;  /* a,b are not used */
  char *Hidden;

  /* seek octets */
  /* Parts[0] = d */
  Parts[0]=HostString;
  /* Parts[1] = c */
  Parts[1]=strchr(Parts[0],'.');
  Parts[1]++;
  /* Parts[2] = domain */
  Parts[2]=strchr(Parts[1],'.');
  Parts[2]++;

  for(d=0; d<256; d++)
    {
    Hidden = OctetLookup(d);
    if (!strncasecmp(Parts[0],Hidden,strlen(Hidden)))
      {
      for(c=0; c<256; c++)
        {
        Hidden = OctetLookup(c);
        if (!strncasecmp(Parts[1],Hidden,strlen(Hidden)))
          {
	  printf("(a.b.%d.%d) in %s\n",c,d,Parts[2]);
	  }
	} /* for c */
      } /* if match d */
    } /* for d */
} /* SeekHost() */

/********************************************************************/
int	main	(int argc, char *argv[])
{
  if (argc != 3)
	{
	printf("USAGE: %s command host\n",argv[0]);
	printf("  commands:\n");
	printf("    hide <host> :: cloak the host (IP or hostname)\n");
	printf("    seekip <host> :: uncloak the encoded IPv4 host\n");
	printf("    seekdomain <host> :: uncloak the encoded hostname (with domain)\n");
	return(0);
	}

  if (!strcasecmp(argv[1],"hide"))
	{
	HideHost(argv[2]);
	}
  if (!strcasecmp(argv[1],"seekip"))
	{
	SeekIp(argv[2]);
	}
  if (!strcasecmp(argv[1],"seekdomain"))
	{
	SeekHost(argv[2]);
	}
  return(0);
} /* main() */

