/*********************************************************
 Hash value computation.
 Copyright 2005 N. Krawetz.  All rights reserved.
 Released as open source.
 No warranty expressed or implied.
 Given input from stdin, compute a variety of checksums.
 *********************************************************/

#include <stdlib.h>
#include <stdio.h>

/**********************************************
 CRC_CCITT_Init(): Initialize the CRC-CCITT table.
 This is a 16-bit CRC.
 **********************************************/
u_int *	CRC_CCITT_Init	()
{
  static u_int CRC_table[256];
  u_int i;
  u_int Bits;
  u_int CRC;

  for(i=0; i<256; i++)
    {
    CRC = (i << 8);
    for(Bits=0; Bits<8; Bits++)
      {
      CRC = CRC * 2;
      if (CRC & 0x10000)  CRC = CRC ^ 0x1021;
      }
    CRC_table[i] = (CRC & 0xffff);
    }

  return(CRC_table);
} /* CRC_CCITT_Init() */

/**********************************************
 CRC_CCITT_compute(): Compute CRC using table.
 This is a 16-bit CRC.
 **********************************************/
u_int CRC_CCITT_Compute	(u_int *Table, u_int CRC, int c)
{
  u_int Bits;

  CRC = ((CRC << 8) ^ Table[ (((CRC >> 8) ^ c) & 0xff)]) & 0xffff;
  return(CRC);
} /* CRC_CCITT_Init() */

/**********************************************
 ComputeParity(): Given a byte and an initial
 parity value, return the new parity value.
 **********************************************/
u_int	ComputeParity	(u_int Parity, u_int Byte)
{
  u_int Sum; /* number of on bits */
  Sum = Parity;
  Sum += ((Byte & 0x80) ? 1 : 0);
  Sum += ((Byte & 0x40) ? 1 : 0);
  Sum += ((Byte & 0x20) ? 1 : 0);
  Sum += ((Byte & 0x10) ? 1 : 0);
  Sum += ((Byte & 0x08) ? 1 : 0);
  Sum += ((Byte & 0x04) ? 1 : 0);
  Sum += ((Byte & 0x02) ? 1 : 0);
  Sum += ((Byte & 0x01) ? 1 : 0);
  if (Sum & 0x01) return(1);	/* odd */
  return(0); /* even */
} /* ComputeParity() */

/************************************************************/
int	main	(int argc, char *argv[])
{
  FILE *Fin;	/* file in */
  int c;	/* character from file */
  int Size;	/* amount of data read from file */
  /* checksum values */
  u_int Parity=0;	/* Parity value */
  u_int Checksum8=0;	/* 8-bit checksum */
  u_int Checksum16=0;	/* 16-bit checksum */
  int Checksum16_buffer;
  /* CRC tables */
  u_int CRC_CCITT=0;
  u_int *CRC_CCITT_Table;

  CRC_CCITT_Table = CRC_CCITT_Init();

  Fin=stdin;
  while(!feof(Fin))
    {
    c = fgetc(Fin);
    if (c >= 0)
      {
      Size++;

      /* Parity */
      Parity=ComputeParity(Parity,c);

      /* 8-bit checksum */
      Checksum8 = (Checksum8 + c) & 0xff;

      /* 16-bit checksum */
      if (Size % 2 == 0)
        {
	Checksum16 = (Checksum16 + Checksum16_buffer*256 + c) & 0xffff;
	}
      else
        {
	Checksum16_buffer = c;
	}

      /* 16-bit CRC */
      CRC_CCITT = CRC_CCITT_Compute(CRC_CCITT_Table,CRC_CCITT,c);
      }
    }

  /* compute any final bits */

  /* Parity: Nothing left to do */
  /* Checksum8: Nothing left to do */
  /* Checksum16: add any extra byte */
      if (Size % 2 != 0)
        {
	/* assume the missing byte of 0x00 */
	Checksum16 = (Checksum16 + Checksum16_buffer*256) & 0xffff;
	}

  printf("Parity = %u\n",Parity);
  printf("8-bit checksum = 0x%02x\n",Checksum8);
  printf("16-bit checksum = 0x%04x\n",Checksum16);
  printf("16-bit CRC = 0x%04x\n",CRC_CCITT);
} /* main() */

