/*********************************************************
 Simple random number generator.
 Copyright 2005 N. Krawetz.  All rights reserved.
 Released as open source.
 No warranty expressed or implied.
 This is a linear congruence pseudo-random number generator.
 Benefits: Fast, simple
 Limitations: Trivially predictable, repetitive
 The algorithm [RFC1750]:
   V[n+1] = (V[n] * a + b)mod c
 *********************************************************/

#include <stdlib.h>
#include <stdio.h>

int	main	(int argc, char *argv[])
{
  unsigned long a,b,c; /* keys for the random generator */
  unsigned long Vold, Vnew; /* values -- the first Vold is the seed */
  unsigned long Num; /* number of values to generate */
  unsigned long i; /* looping variable */

  /* check parameters */
  if (argc != 6)
    {
    printf("Usage: %s num seed a b c\n",argv[0]);
    printf("  num = number of random values to generate\n");
    printf("  seed = seed for random number generator\n");
    printf("  a = multiplier for random number generator\n");
    printf("  b = offset for random number generator\n");
    printf("  c = modulus for random number generator\n");
    return(-1);
    }

  /* init values */
  Num = atol(argv[1]);
  Vold = atol(argv[2]);
  a = atol(argv[3]);
  b = atol(argv[4]);
  c = atol(argv[5]);

  /* generate numbers */
  for(i=0; i<Num; i++)
    {
    Vnew = ( (Vold * a) + b ) % c;
    printf("%lu\n",Vnew);
    Vold = Vnew;
    }
  return(0);
} /* main() */

