/**************************************************************
 WEP Password: generate a web password hash.
 Copyright 2005 N. Krawetz.  All rights reserved.
 Released as open source.
 No warranty expressed or implied.
 **************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include "md5.h"

/*********************************************
 Create40bit(): Given a string, generate a 40-bit
 (64bit WEP) public key.  Display the key in hex.
 4 keys are created for every 1 password.
 *********************************************/
void	Create40bit	(char *Text)
{
  int i,j;
  int Seed[4] = {0,0,0,0};
  int TextLen;
  unsigned int Rand=0;
  unsigned int Value;


  /* Create the seed based on the string */
  TextLen = strlen(Text);
  for (i=0; i<TextLen; i++)	Seed[i%4] ^= Text[i];

  /* Create the initial random number generator */
  Rand = Seed[0] | (Seed[1] << 8) | (Seed[2] << 16) | (Seed[3] << 24);

  /* Create the key */
  for (i=0; i<4; i++)  /* create 4 keys */
    {
    printf("  Key #%d: ",i);
    for (j=0; j<5; j++)  /* 5 bytes x 8 bits = 40 bits */
      {
      Rand = Rand * 0x343fd + 0x269ec3;
      Value = (Rand >> 16) & 0xff;
      printf("%02x",Value);
      }
    printf("\n");
    }
} /* Create40bit() */

/*********************************************
 Create104bit(): Given a string, generate a 104-bit
 (128bit WEP) public key.  Display the key in hex.
 1 keys is created for every string.
 Unlike 40bit WEP, this uses MD5 as a hash function.
 *********************************************/
void	Create104bit	(char *Text)
{
  char PasswordBuffer[65];
  int i;
  int TextLen;
  MD5_CTX Context;	/* the MD5 state machine (context) */
  unsigned char Result[16];

  /* Initialize the password by concatenating the same string
     until we reach 64 bytes */
  TextLen = strlen(Text);
  for(i=0; i<64; i++)	PasswordBuffer[i] = Text[i%TextLen];

  /* Initialize the MD5 context */
  MD5_Init(&Context);

  /* Create the 128-bit signature */
  MD5_Update(&Context,PasswordBuffer,64);
  MD5_Final(Result,&Context);

  /* Display only the first 104 bits (13 bytes) */
  printf("  Key: ");
  for(i=0; i<13; i++)	printf("%02x",Result[i]);
  printf("\n");
} /* Create104bit() */

/***************************************************
 main(): The exciting user interface.
 ***************************************************/
int	main	(int argc, char *argv[])
{
  if ((argc != 2) || (strlen(argv[1]) < 1))
    {
    printf("Usage: %s password\n",argv[0]);
    printf("  This program will display the WEP keys for the password.\n");
    printf("  Both 64bit and 128bit keys are displayed.\n");
    exit(-1);
    }

  printf("There are four 64bit WEP keys:\n");
  Create40bit(argv[1]);
  printf("There is one 128bit WEP key:\n");
  Create104bit(argv[1]);
  return(0);
} /* main() */

