/*****************************************************************
 Get MAC
 Copyright 2005 N. Krawetz  All rights reserved.

 Purpose:
   List all MAC addresses and IP addresses.

 Usage:
   getmac
 *****************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <sys/ioctl.h>
#include <net/if.h>

/*********************************************************
 ListInterface(): A computer may have multiple network cards.
 List each interface, hardware address, and IP address.
 *********************************************************/
void	ListInterface	()
{
  struct ifreq Interface;
  struct ifreq *InterfacePointer;	/* pointer to an interface */
  struct ifconf InterfaceConfig;
  char Buffer[0x8000];	/* for interface configuration (make it big) */
  int i;
  int Socket;

  /* Prepare a socket */
  /*** Any socket will work, don't require root ***/
  Socket = socket(AF_INET,SOCK_STREAM,0);
  if (Socket < 0)
    {
    printf("ERROR: Failed to open socket.\n");
    exit(-1);
    }

  /** SIOCGIFCONF returns list of interfaces **/
  InterfaceConfig.ifc_len = sizeof(Buffer);
  InterfaceConfig.ifc_buf = Buffer;
  if (ioctl(Socket,SIOCGIFCONF,&InterfaceConfig) < 0)
    {
    printf("Error obtaining %s IP address\n",Interface.ifr_name);
    exit(-1);
    }
  /** list every interface **/
  for(i=0; i*sizeof(struct ifreq) < InterfaceConfig.ifc_len; i++)
    {
    InterfacePointer=&(InterfaceConfig.ifc_req[i]);

    /* Load the interface mac address */
    strcpy(Interface.ifr_name,InterfacePointer->ifr_name);

    ioctl(Socket,SIOCGIFHWADDR,&Interface);

    /* Display the results */
    /** The "unsigned int" and "0xff" are just so negative characters
        appear as positive values between 0 and 255. **/
    printf("Interface: %-6s  ",InterfacePointer->ifr_name);
    printf("MAC: %02x:%02x:%02x:%02x:%02x:%02x  ",
	(unsigned int)(Interface.ifr_hwaddr.sa_data[0])&0xff,
	(unsigned int)(Interface.ifr_hwaddr.sa_data[1])&0xff,
	(unsigned int)(Interface.ifr_hwaddr.sa_data[2])&0xff,
	(unsigned int)(Interface.ifr_hwaddr.sa_data[3])&0xff,
	(unsigned int)(Interface.ifr_hwaddr.sa_data[4])&0xff,
	(unsigned int)(Interface.ifr_hwaddr.sa_data[5])&0xff
	);
    printf("IP: %u.%u.%u.%u\n",
	(unsigned int)(InterfacePointer->ifr_addr.sa_data[2])&0xff,
	(unsigned int)(InterfacePointer->ifr_addr.sa_data[3])&0xff,
	(unsigned int)(InterfacePointer->ifr_addr.sa_data[4])&0xff,
	(unsigned int)(InterfacePointer->ifr_addr.sa_data[5])&0xff
	);
    }
  close(Socket);
} /* ListInterface() */

/*********************************************************
 Usage(): Display program usage.
 *********************************************************/
void	Usage	(char *Name)
{
  printf("Usage: %s\n",Name);
  printf("  This will list ever interface, MAC address, and IP address.\n");
} /* Usage() */

/*********************************************************/
int	main	(int argc, char *argv[])
{
  if (argc != 1)
  	{
	Usage(argv[0]);
	exit(-1);
	}
  ListInterface();
  return(0);
} /* main() */

