/************************************************************************
 ListHosts: Given an IP range, list all resolvable hostnames in that range.
 Copyright 2003-2004 Hacker Factor, All rights reserved.
 ************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <sys/stat.h>
#include <string.h>
#include <limits.h>
/* for inet_addr */
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
/* for gethostbyname() */
#include <netdb.h>

/**************************************************
 ListRange(): Scan a range of IP addresses.
 List any that resolve.
 **************************************************/
void	ListRange	(uint32_t Start, uint32_t Stop, int Sleep, int Count)
{
  int i;
  char *S, AddrIP[100];
  struct hostent *Hent;
  struct in_addr AddrIn;
  int Counter=0;

  /* process each IP address */
  sethostent(1);
  for( ; Start <= Stop ; Start++)
    {
    memset(AddrIP,0,sizeof(AddrIP));
    AddrIn.s_addr = htonl(Start);
    strcpy(AddrIP,inet_ntoa(AddrIn));
    Hent = gethostbyaddr((char *)&AddrIn,sizeof(AddrIn),AF_INET);
    if (Hent)	S = Hent->h_name;
    else	S = NULL;
    if (S)
    	{
	printf("%s %s\n",AddrIP,S);
	/* check every alias */
	for(i=0; Hent->h_aliases[i]; i++)
	  {
	  S = Hent->h_aliases[i];
	  printf("          %s\n",S);
	  fflush(stdout);
	  }
	}
    Counter++;
    if (Counter >= Count)
      {
      Counter=0;
      if (Sleep > 0)
      	{
	sleep(Sleep);
	}
      }
    } /* for() */
  endhostent();
} /* ListRange() */

/**************************************************
 Usage(): To avoid confusion...
 **************************************************/
void	Usage	(char *Name)
{
  printf("Usage:\n");
  printf("  %s ip_start ip_stop [delay [requests]]\n",Name);
  printf("       List all resolvable hostnames in the range.\n");
  printf("       This is as slow as your DNS (it does a lookup each time).\n");
  printf("       Some DNS servers may lock you out (too many requests).\n");
  printf("       The optional delay tells how many seconds to pause.\n");
  printf("       Be default, the pause is after every 256 (Class-C) requests.\n");

} /* Usage() */

/*************************************************************************/
int	main	(int argc, char *argv[])
{
  int Sleep, Count;
  Sleep=0;
  Count=256;

  /* idiot checking and memory init */
  if ((argc < 3) || (argc > 5))
	{
	Usage(argv[0]);
	return(0);
	}
  if (argc > 3) Sleep=atoi(argv[3]);
  if (argc > 4) Count=atoi(argv[4]);
  ListRange(ntohl(inet_addr(argv[1])),ntohl(inet_addr(argv[2])),Sleep,Count);
  return(0);
} /* main() */

